import java.io.*;
import java.math.*;
import java.util.*;

public class Validate {
    private static final int MAXLEN = 200000;
    private static final int MAXN = 100000;

    public void run() {
        StrictScanner inf = new StrictScanner(System.in);
        String expression = inf.next();
        ensureLimits(expression.length(), 1, MAXLEN, "Wrong length of expression -");
        inf.nextLine();
        try {
            ExpressionParser parser = new ExpressionParser();
            int result = parser.parse(expression);
        } catch (ParserException e) {
            ensure(false, e.getMessage());
        }                                          
        int n = inf.nextInt();
        ensureLimits(n, 0, MAXN, "Wrong n -");
        inf.nextLine();
        for (int i = 0; i < n; i++) {
            int index = inf.nextInt();
            inf.nextLine();
            ensureLimits(index, 1, expression.length(), "Wrong index #" + (i + 1) + " - expected from 1 to " + expression.length() + ", but found " + index);
            ensure(expression.charAt(index - 1) == '+' || expression.charAt(index - 1) == '*', "Wrong query index #" + (i + 1) + " - " + expression.charAt(index - 1));
        }
        inf.close();
    }

    private class ParserException extends Exception {
        public ParserException(String message) {
            super(message);
        }
    }

    private class ExpressionParser {
        private static final long MOD = 1_000_000_007;
        
        private String expression;
        private int index;
        private long value;

        private boolean isDigit(char c) {
            return '0' <= c && c <= '9';
        }

        private void skipSpaces() {
            while (index < expression.length() && Character.isWhitespace(expression.charAt(index))) {
                index++;
            }
        }
        
        private void parseInt() throws ParserException {
            value = 0;
            if (expression.charAt(index) == '0') {
                throw new ParserException("Leading zero");
            }
            while (index < expression.length() && isDigit(expression.charAt(index))) {
                value = (value * 10 + expression.charAt(index) - '0') % MOD;
                index++;
            }
        }

        private int parseMultiplier() throws ParserException {
            skipSpaces();
            if (index >= expression.length()) {
                throw new ParserException("Unexpected end of expression");
            }
            if (isDigit(expression.charAt(index))) {
                parseInt();
                skipSpaces();
                return (int)value;
            } else if (expression.charAt(index) == '(') {
                index++;
                int result = parseExpression();
                if (index >= expression.length() || expression.charAt(index) != ')') {
                    throw new ParserException("No pair for open parenthesis");
                }
                index++;
                skipSpaces();
                return result;
            } 
            throw new ParserException("Wrong symbol at position " + (index + 1) + ": " + expression.charAt(index));
        }

        private int parseSummand() throws ParserException {
            int result = parseMultiplier();
            skipSpaces();
            while (index < expression.length() && expression.charAt(index) == '*') {
                index++;
                result *= parseMultiplier();
                skipSpaces();
            }
            return result;
        }

        private int parseExpression() throws ParserException {
            int result = parseSummand();
            skipSpaces();
            while (index < expression.length() && expression.charAt(index) == '+') {
                index++;
                result += parseSummand();
                skipSpaces();
            }
            return result;
        }

        public int parse(String expression) throws ParserException {
            this.expression = expression;
            value = 0;
            index = 0;
            int result = parseExpression();
            if (index != expression.length()) {
                throw new ParserException("Extra information at the end of expression");
            }
            return result;
        }
    }
    
    public static void main(String[] args) {
        new Validate().run();
    }

    public class StrictScanner {
        private final BufferedReader in;
        private String line = "";
        private int pos;
        private int lineNo;

        public StrictScanner(InputStream source) {
            in = new BufferedReader(new InputStreamReader(source));
            nextLine();
        }

        public void close() {
            ensure(line == null, "Extra data at the end of file");
            try {
                in.close();
            } catch (IOException e) {
                throw new AssertionError("Failed to close with " + e);
            }
        }

        public void nextLine() {
            ensure(line != null, "EOF");
            ensure(pos == line.length(),  "Extra characters on line " + lineNo);
            try {
                line = in.readLine();
            } catch (IOException e) {
                throw new AssertionError("Failed to read line with " + e);
            }
            pos = 0;
            lineNo++;
        }

        public String next() {
            ensure(line != null, "EOF");
            ensure(line.length() > 0,  "Empty line " + lineNo);
            StringBuilder sb = new StringBuilder();
            while (pos < line.length())
                sb.append(line.charAt(pos++));
            return sb.toString();
        }

        public int nextInt() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',  "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+',  "Extra leading '+' in number " + s + " on line " + lineNo);
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }

        public long nextLong() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Long.parseLong(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public double nextDouble() {
            String s = next();
            ensure(s.length() == 1 || s.startsWith("0.") || s.charAt(0) != '0',  "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+',  "Extra leading '+' in number " + s + " on line " + lineNo);
            try {
                return Double.parseDouble(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }
    }

    void ensure(boolean b, String message) {
        if (!b) {
            throw new AssertionError(message);
        }
    }
    
    void ensureLimits(int n, int from, int to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to " + to);
    }

    void ensureLimits(long n, long from, long to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to " + to);
    }
}