import javax.swing.event.TreeExpansionEvent;
import java.io.IOException;
import java.lang.*;
import java.io.*;
import java.nio.file.Paths;
import java.util.*;
import java.lang.String;

public class TestGen {
    private int testCount;
    private final Random rnd = new Random(0xABACABA);

    private static final class Tree {
        Map<String, Tree> children;
        int size;

        public Tree() {
            children = new HashMap<>();
        }
    }

    private String genRandomName(int from, int to) {
        int len = rnd.nextInt(to - from) + from;
        char[] res = new char[len];
        for (int i = 0; i < len; i++) {
            res[i] = randomChar();
        }
        return new String(res);
    }

    private char randomChar() {
        // 'a'..'z' + 'A'..'Z' + '.'
        int code = rnd.nextInt(26 * 2 + 1);
        if (code < 26) {
            return (char) ('a' + code);
        } else if (code < 26 * 2) {
            return (char) ('A' + code - 26);
        } else {
            return '.';
        }
    }

    private void dfs(PrintWriter out, String prefix, Tree tree) {
        if (tree == null) {
            return;
        }
        for (String name : tree.children.keySet()) {
            String newPrefix = (prefix == null ? "" : prefix + "/") + name;
            out.println(newPrefix);
            dfs(out, newPrefix, tree.children.get(name));
        }
    }

    private void writeTree(PrintWriter out, Tree tree) {
        dfs(out, null, tree);
    }

    private void dfsWithSkips(List<String> out, String prefix, Tree tree, float prob) {
        if (tree == null) {
            return;
        }
        for (String name : tree.children.keySet()) {
            String newPrefix = (prefix == null ? "" : prefix + "/") + name;
            if (rnd.nextInt(100000) < prob * 100000) {
                out.add(newPrefix);
            }
            dfsWithSkips(out, newPrefix, tree.children.get(name), prob);
        }
    }

    private void writeTreeWithSkips(List<String> out, Tree tree, float prob) {
        dfsWithSkips(out, null, tree, prob);
    }

    private Tree genTree(int n) {
        n++;
        int[] p = new int[n];
        for (int i = 1; i < n; i++) {
            p[i] = rnd.nextInt(i);
        }
        String[] names = genRandomNames(n);
        Tree[] trees = new Tree[n];
        for (int i = 0; i < n; i++) {
            trees[i] = new Tree();
        }
        for (int i = 1; i < n; i++) {
            trees[p[i]].children.put(names[i - 1], trees[i]);
        }
        trees[0].size = n - 1;
        return trees[0];
    }

    private Tree genBinaryTree(int n) {
        n++;
        int[] p = new int[n];
        for (int i = 1; i < n; i++) {
            p[i] = i >> 1;
        }
        String[] names = genRandomNames(n);
        Tree[] trees = new Tree[n];
        for (int i = 0; i < n; i++) {
            trees[i] = new Tree();
        }
        for (int i = 1; i < n; i++) {
            trees[p[i]].children.put(names[i - 1], trees[i]);
        }
        trees[0].size = n - 1;
        return trees[0];
    }

    private void genRandomTree(int n) {
        Tree data = genTree(n);
        write(new TestWriter() {
            @Override
            public void write(PrintWriter out) {
                out.println(data.size);
                writeTree(out, data);
            }
        });
    }

    private void genRandomBinaryTree(int n) {
        Tree data = genBinaryTree(n);
        write(new TestWriter() {
                @Override
                public void write(PrintWriter out) {
                    out.println(data.size);
                    writeTree(out, data);
                }
            });
    }

    private void genRandomTreeWithSkips(int n, float prob) {
        List<String> ans = new ArrayList<>();
        writeTreeWithSkips(ans, genTree(n), prob);
        write(new TestWriter() {
            @Override
            public void write(PrintWriter out) {
                out.println(ans.size());
                for (String s : ans) {
                    out.println(s);
                }
            }
        });
    }

    private String[] genRandomNames(int n) {
        String[] names = new String[n - 1];
        for (int i = 0; i < n - 1; i++) {
            names[i] = genRandomName(5, 10);
        }
        return names;
    }

    interface TestWriter {
        void write(PrintWriter out);
    }

    private void write(TestWriter writer) {
        String path = String.format("../%s/%02d", "tests", ++testCount);
        PrintWriter out = null;
        try {
            out = new PrintWriter(path);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            System.exit(1);
        }
        writer.write(out);

        out.close();
        System.out.println(String.format("Test %d is generated", testCount));
    }

    private void writeFromFile(String path) {
        write(new TestWriter() {
            @Override
            public void write(PrintWriter out) {
                try(FileInputStream is = new FileInputStream(path)) {
                    Scanner in = new Scanner(is);
                    while (in.hasNextLine()) {
                        out.println(in.nextLine());
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    System.exit(1);
                }
            }
        });
    }

    private void run() throws IOException {
        //noinspection ResultOfMethodCallIgnored
        new File("../tests").mkdir();

        genSamples();
        genRandomTree(5);
        genRandomTree(100);
        genRandomTree(100);
        genRandomTree(100);
        genRandomTree(100);

        genRandomTreeWithSkips(100, 0.1f);
        genRandomTreeWithSkips(100, 0.2f);
        genRandomTreeWithSkips(100, 0.3f);
        genRandomTreeWithSkips(100, 0.4f);
        genRandomTreeWithSkips(100, 0.5f);
        genRandomTreeWithSkips(100, 0.6f);
        genRandomTreeWithSkips(100, 0.7f);
        genRandomTreeWithSkips(100, 0.8f);
        genRandomTreeWithSkips(100, 0.9f);

        genRandomBinaryTree(100);
        genRandomBinaryTree(100);
        genRandomBinaryTree(100);

        genMax(100);
        genMax(100);
        genMax(100);

        genMax2(100);
        genMax2(100);
        genMax2(100);
    }

    private void genMax2(int n) {
        write(new TestWriter() {
            @Override
            public void write(PrintWriter out) {
                out.println(n);
                for (int i = 0; i < n; i++) {
                    out.println(genRandomName(n, n + 1));
                }
            }
        });
    }

    private void genMax(int n) {
        write(new TestWriter() {
            @Override
            public void write(PrintWriter out) {
                out.println(n);
                for (int j = 0; j < n; j++) {
                    for (int i = 0; i < n / 2 - 1; i++) {
                        if (i > 0) out.print('/');
                        out.print(randomChar());
                    }
                    out.println();
                }
            }
        });
    }

    private void genSamples() {
        writeFromFile("sample1.in");
        writeFromFile("sample2.in");
        writeFromFile("sample3.in");
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
